@@ -7,17 +7,11 @@ see "Snap.Snaplet".  For the core web server API, see "Snap.Core".
 -}
 
 module Snap
-  ( module Control.Applicative
-  , module Control.Lens.Loupe
-  , module Control.Monad.State
-  , module Snap.Core
+  ( module Snap.Core
   , module Snap.Http.Server
   , module Snap.Snaplet
   ) where
 
-import Control.Applicative
-import Control.Lens.Loupe
-import Control.Monad.State
 import Snap.Core
 import Snap.Http.Server
 import Snap.Snaplet
